'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isScalar = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _storableException = require('./exception/storableException');

var _storableException2 = _interopRequireDefault(_storableException);

var _storableLink = require('./storableLink');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Return true if the parameter is scalar
 * @param obj
 */
var isScalar = exports.isScalar = function isScalar(obj) {
  return (/string|number|boolean/.test(typeof obj === 'undefined' ? 'undefined' : _typeof(obj))
  );
};
var checkObjectValues = function checkObjectValues(object) {
  Object.keys(object).forEach(function (key) {
    if (!isScalar(object[key]) && !(0, _storableLink.isStorableLink)(object[key]) && object[key] !== null) {
      if (_typeof(object[key]) === "object" && !Array.isArray(object[key])) {
        checkObjectValues(object[key]);
      } else {
        throw (0, _storableException2.default)("A storable object must only have scalar values. Check the value for key " + key);
      }
    }
  });
};

/**
 * Create a Storable from the given object.
 * @param object
 * @returns {Object}
 */
var createStorable = function createStorable(object) {
  if ((typeof object === 'undefined' ? 'undefined' : _typeof(object)) !== "object") {
    throw (0, _storableException2.default)("The parameter of createStorable must be an object");
  }

  if (!object.hasOwnProperty("id")) {
    throw (0, _storableException2.default)("A storable must have an ID.");
  }

  checkObjectValues(object);

  return function (entity) {
    var _data = Object.assign({}, entity);

    var obj = {
      getData: function getData() {
        return _data;
      },
      merge: function merge(data) {
        if (data.id) {
          throw (0, _storableException2.default)("You can't supply an ID in merge.");
        }

        _data = Object.assign({}, _data, data);
      },
      clone: function clone() {
        return createStorable(_data);
      }
    };

    return obj;
  }(object);
};

var storableFactory = {
  createStorable: createStorable
};

exports.default = storableFactory;