'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

/**
 * Representa oneToOne relationship
 * @param container
 * @param store
 * @param id
 * @returns {{getStorable: (function())}}
 */
var toOne = exports.toOne = function toOne(container, store, id) {
  return {
    getStorable: function getStorable() {
      return container.getStore(store).get(id);
    }
  };
};

/**
 * Represents a oneToMany relationship
 *  @param container
 * @param store
 * @param ids
 * @returns {{getStorable: (function(): Array)}}
 */
var toMany = exports.toMany = function toMany(container, store, ids) {
  return {
    getStorable: function getStorable() {
      return ids.map(function (id) {
        return container.getStore(store).get(id);
      });
    }
  };
};

/**
 * Return true if the given value is a storableLink
 * @param value
 */
var isStorableLink = exports.isStorableLink = function isStorableLink(value) {
  return value && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === "object" && value.getStorable && typeof value.getStorable === "function";
};
exports.default = {
  toOne: toOne,
  toMany: toMany
};