'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _clone = require('clone');

var _clone2 = _interopRequireDefault(_clone);

var _storable = require('./storable');

var _storable2 = _interopRequireDefault(_storable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var cloneArrayOfStorable = function cloneArrayOfStorable(state) {
  return state.map(function (storable) {
    return storable.clone();
  });
};

var createStore = function createStore(name) {
  var initialState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

  var items = cloneArrayOfStorable(initialState);
  var storeInitialState = cloneArrayOfStorable(initialState);

  var store = {
    /**
     * @var string
     */
    name: name,

    /**
     * Restore the initial state of the store
     * @returns {StoreType}
     */
    reset: function reset() {
      items = cloneArrayOfStorable(storeInitialState);
      return store;
    },

    /**
     * Add a storable to the store
     * @param entity
     * @returns {StoreType}
     */
    addStorable: function addStorable(entity) {
      items = [].concat(_toConsumableArray(items), [entity]);

      return store;
    },

    /**
     * Convert an object into a storable and add it to the store
     * @param entity
     * @returns {StoreType}
     */
    add: function add(entity) {
      items = [].concat(_toConsumableArray(items), [_storable2.default.createStorable(entity)]);

      return store;
    },

    /**
     * Remove the storable from the store
     * @param id
     * @returns {StoreType}
     */
    remove: function remove(id) {
      items = items.filter(function (entity) {
        return entity.getData().id !== id;
      });
      return store;
    },

    /**
     * Update the storable with a new storable
     * @param id
     * @param next
     * @returns {StoreType}
     */
    update: function update(id, next) {
      items = items.map(function (storable) {
        if (storable.getData().id !== id) {
          return storable;
        }

        return next;
      });

      return store;
    },

    /**
     * Return the storable having the given ID
     * @param id
     * @returns {StorableType}
     */
    get: function get(id) {
      return items.find(function (entity) {
        return entity.getData().id === id;
      });
    },

    /**
     * Return the store
     * @returns {Array<StorableType>}
     */
    all: function all() {
      return items;
    },

    /**
     * Create a storable
     * @returns {StorableType}
     */
    createStorable: _storable2.default.createStorable
  };

  return store;
};

exports.default = {
  createStore: createStore,
  createStorable: _storable2.default.createStorable
};